<div {% if buttonholder.css_id %}id="{{ buttonholder.css_id }}"{% endif %} 
    class="buttonHolder{% if buttonholder.css_class %} {{ buttonholder.css_class }}{% endif %}">
       {{ fields_output|safe }}
</div>
